MessageNode subclass: #ObjectLispAskMessageNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Prototypes-ProgramNodes'!


!ObjectLispAskMessageNode methodsFor: 'evaluator'!

eval: context 
	"context contains self which is the initial receiver in the current delegation 
	chain, that is the client and curRec the current receiver."

	|  rec  newContext  result |
	rec := receiver eval: context.
	newContext := PContext new.
	newContext at: #self put: rec.
	arguments do: [:each | result := each eval: newContext].
	^result! !

AssignmentNode subclass: #EncapsulationAssignmentNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Prototypes-ProgramNodes'!


!EncapsulationAssignmentNode methodsFor: 'evaluator'!

eval: context 
	| varName val   |
	varName := variable name asSymbol.
	val := value eval: context.
	^(context hasLocalVar: varName)
		ifTrue: [context at: varName put: val]
		ifFalse: [(context at: #self)
				var: varName put: val]!

evalInternal: context 
	| varName val   |
	varName := variable name asSymbol.
	val := value evalInternal: context.
	^(context hasLocalVar: varName)
		ifTrue: [context at: varName put: val]
		ifFalse: [(context at: #self)
				var: varName put: val]! !

ProgramNodeBuilder subclass: #ImplicitDelegationPNB
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Prototypes-ProgramNodes'!


!ImplicitDelegationPNB methodsFor: 'node creation-basic'!

newMessageReceiver: rcvr selector: sel arguments: args 
	((rcvr isKindOf: VariableNode)
		and: [rcvr name = #super])
		ifTrue: [^SuperMessageNode new
				receiver: rcvr
				selector: sel
				arguments: args]
		ifFalse: [^ImplicitDelegationMN new
				receiver: rcvr
				selector: sel
				arguments: args]! !

ImplicitDelegationPNB subclass: #ImplicitDelegationEncapsulationPNB
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Prototypes-ProgramNodes'!


!ImplicitDelegationEncapsulationPNB methodsFor: 'node creation-basic'!

newAssignmentVariable: var value: val leftArrow: bool
	^EncapsulationAssignmentNode new variable: var value: val leftArrow: bool!

newVariableName: nameString
	^EncapsulationVariableNode new name: nameString! !

ImplicitDelegationEncapsulationPNB subclass: #ObjectLispLikePNB
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Prototypes-ProgramNodes'!


!ObjectLispLikePNB methodsFor: 'node creation-basic'!

newMessageReceiver: rcvr selector: sel arguments: args 
	sel = #ask:
		ifTrue: [^ObjectLispAskMessageNode new
				receiver: rcvr
				selector: sel
				arguments: args]
		ifFalse: [^super
				newMessageReceiver: rcvr
				selector: sel
				arguments: args]! !

ImplicitDelegationPNB subclass: #SelfLikePNB
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Prototypes-ProgramNodes'!


!SelfLikePNB methodsFor: 'node creation-basic'!

newVariableName: nameString
	^SelfLikeVariableNode new name: nameString! !

MessageNode subclass: #ImplicitDelegationMN
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Prototypes-ProgramNodes'!


!ImplicitDelegationMN methodsFor: 'proto-evaluator'!

eval: context 
	"context contains self which is the initial receiver in the current delegation 
	chain"

	| method rec args newContext owner |
	rec := receiver eval: context.
	args := arguments evlis: context.
	owner := rec lookup: selector.
	owner isNil ifTrue: [AbstractProto unknownMethod raiseWith: selector].
	method := owner compiledMethodAt: selector.
	(method isKindOf: CompiledMethod)
		ifTrue: ["this is either a call to a primitive or a smalltalk message"
			^rec perform: selector withArguments: args].
	"this is a user-defined method"
	newContext := PContext new.
      newContext at: #self put: rec.
	newContext at: #super put: (owner parent).
	^method applyWith: args in: newContext! !

ProgramNodeBuilder subclass: #ExplicitDelegationPNB
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Prototypes-ProgramNodes'!


!ExplicitDelegationPNB methodsFor: 'node creation-basic'!

newMessageReceiver: rcvr selector: sel arguments: args 
	sel = #delegates:withArgs:
		ifTrue: [^ExplicitDelegationMN new
				receiver: rcvr
				selector: (args at: 1) value
				arguments: Array new"ICI a corriger"]
		ifFalse: [^super
				newMessageReceiver: rcvr
				selector: sel
				arguments: args]! !

ExplicitDelegationPNB subclass: #ExplicitDelegationEncapsulationPNB
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Prototypes-ProgramNodes'!
ExplicitDelegationEncapsulationPNB comment:
'With multiple inheritance, this class should inherit from ExplicitDelegationPNB and EncapsulationPNB. 
The second one is simulated.'!


!ExplicitDelegationEncapsulationPNB methodsFor: 'node creation-basic'!

newAssignmentVariable: var value: val leftArrow: bool
	^EncapsulationPNB new newAssignmentVariable: var value: val leftArrow: bool!

newVariableName: nameString
	^EncapsulationVariableNode new name: nameString! !

ProgramNodeBuilder subclass: #EncapsulationPNB
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Prototypes-ProgramNodes'!


!EncapsulationPNB methodsFor: 'node creation-basic'!

newAssignmentVariable: var value: val leftArrow: bool
	^EncapsulationAssignmentNode new variable: var value: val leftArrow: bool!

newVariableName: nameString
	^EncapsulationVariableNode new name: nameString! !

VariableNode subclass: #SelfLikeVariableNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Prototypes-ProgramNodes'!


!SelfLikeVariableNode methodsFor: 'evaluator'!

eval: context 
	"With Self, each variable node has to be interpreted as a message to 
	the implicit receiver self or as an access to a global variable"

	| client varName owner |
	varName := name asSymbol.
	^context at: varName
		ifAbsent: 
			[client := context at: #self.
			owner := client methodOwner: varName.
			owner isNil
				ifTrue: [client globalVarValue: varName]
				ifFalse: 
					[| method |
					method := owner compiledMethodAt: varName.
					method applyWith: #() in: context]]! !

MessageNode subclass: #SuperMessageNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Prototypes-ProgramNodes'!


!SuperMessageNode methodsFor: 'evaluator'!

eval: context 
	"context contains self which is the initial receiver in the current delegation 
	chain, that is the client and curRec the current receiver."

	| method rec args newContext owner |
	rec := receiver eval: context.
	args := arguments evlis: context.
	owner := rec lookup: selector.
	owner isNil ifTrue: [AbstractProto unknownMethod raiseWith: selector].
	method := owner compiledMethodAt: selector.
	newContext := PContext new.
	newContext at: #self put: (context at: #self).
	newContext at: #super put: (owner parent).
	^method applyWith: args in: newContext! !

VariableNode subclass: #EncapsulationVariableNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Prototypes-ProgramNodes'!


!EncapsulationVariableNode methodsFor: 'evaluator'!

eval: context 
	|  varName  |
	varName := name asSymbol.
	^context at: varName
		ifAbsent: 
			[(context at: #self) varValue: varName]!

evalInternal: context 
	|  varName  |
	varName := name asSymbol.
	^context at: varName
		ifAbsent: 
			[(context at: #self) varValue: varName]! !

MessageNode subclass: #ExplicitDelegationMN
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Prototypes-ProgramNodes'!


!ExplicitDelegationMN methodsFor: 'evaluator'!

eval: context 
	"context contains self which is the initial receiver in the current delegation 
	chain, rec is the object that receives the delegation message."

	| method rec args newContext |
	rec := receiver eval: context.
	args := arguments evlis: context.
	method := rec methodNamed: selector.
	method isNil ifTrue: [AbstractProto unknownMethod raiseWith: selector].
	newContext := PContext new.
	newContext at: #self put: (context at: #self).
	^method applyWith: args in: newContext! !

ProgramNodeBuilder subclass: #SelfLikeAssignmentMethodBuilder
	instanceVariableNames: 'accessPrimName '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Prototypes-ProgramNodes'!


!SelfLikeAssignmentMethodBuilder methodsFor: 'node creation-basic'!

newMethodSelector: sel primitive: prim block: block
	^(SelfLikeAssignmentMethodNode new)
		selector: sel
		primitive: prim
		block: block; accessPrimName: accessPrimName! !

!SelfLikeAssignmentMethodBuilder methodsFor: 'accessing'!

accessPrimName: x
	accessPrimName := x! !

MethodNode subclass: #SelfLikeAssignmentMethodNode
	instanceVariableNames: 'accessPrimName '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Prototypes-ProgramNodes'!


!SelfLikeAssignmentMethodNode methodsFor: 'evaluator'!

applyWith: args in: context 
	(((context at: #self)
		methodOwner: accessPrimName)
		compiledMethodAt: accessPrimName) block body statements first value: (args at: 1).
	^context at: #self! !

!SelfLikeAssignmentMethodNode methodsFor: 'accessing'!

accessPrimName: x
	accessPrimName := x! !
