StringHolderController subclass: #ProtoSpaceController
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Prototypes-Tools'!


!ProtoSpaceController methodsFor: 'menu messages'!

doIt
	"Treat the current text selection as an expression; evaluate it"
	^self view toplevelObject class evaluate: self selectionAsStream for: self!

inspectIt
	self doIt inspect!

printIt
	"Treat the current text selection as an expression; evaluate it. Insert 
	the description of the result of evaluation after the selection and then make 
	this 
	description the new text selection."

	| result |
	result := self doIt.
	result ~~ #failedDoit
		ifTrue: 
			[| aStream method args |
			aStream := WriteStream on: (String new: 16).
			method := (result methodOwner: #printOn:)
						compiledMethodAt: #printOn:.
			args := Array with: aStream.
			(method isKindOf: CompiledMethod)
				ifTrue: [result perform: #printOn: withArguments: args]
				ifFalse: 
					[| newContext |
					newContext := PContext new.
					newContext at: #self put: result.
					method applyWith: args in: newContext].
			self afterSelectionInsertAndSelect: aStream contents]! !

Object variableSubclass: #DelegatesTo
	instanceVariableNames: 'server '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Prototypes-Tools'!


!DelegatesTo methodsFor: 'accessing'!

server
	^server!

server: anObject
	server := anObject! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

DelegatesTo class
	instanceVariableNames: ''!


!DelegatesTo class methodsFor: 'creation'!

server: anObject
	^self new server: anObject! !

Browser subclass: #BasicProtoMethodBrowser
	instanceVariableNames: 'proto externalName '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Prototypes-Tools'!


!BasicProtoMethodBrowser methodsFor: 'reuse interface'!

selectedClass
	"should return the currently browsed prototype"
	^proto! !

!BasicProtoMethodBrowser methodsFor: 'protocole messages'!

addProtocol
	| aString newProtocol |
	self changeRequest ifFalse: [^self].
	aString := self
			prompt: 'Enter new protocol name'
			initially: (LastProtocol == nil ifTrue: ['protocol name'] ifFalse: [LastProtocol]).
	aString isEmpty ifTrue: [^ self].
	newProtocol := aString asSymbol.
	proto organization addCategory: newProtocol before: protocol.
	self newProtocolList: newProtocol!

fileOutProtocol

	| fileName  fileStream |
	externalName isNil ifTrue: [self externalName: (self prompt: 'Type prototype external name' initially: '')].
	fileName := Filename request: 'File out on' initially: (externalName , '-' , protocol printString , '.st') shouldExist: false. 
	fileName = '' ifTrue: [^nil].
	fileStream := (Filename named: fileName) writeStream.
	[proto
		fileOutProtocol: protocol
		file: fileStream
		name: externalName]
		valueNowOrOnUnwindDo: [fileStream close]!

protocolMenu
	"Answer a Menu of operations on message categories to be 
	displayed when the operate menu button is pressed."

	"Browser flushMenus"
	protocol == nil ifTrue:
		[^ PopUpMenu labels: 'add protocol' values: #(addProtocol)].
	ProtocolMenu == nil ifTrue:
		[ProtocolMenu := PopUpMenu
			labels: 'file out\add protocol\rename\remove' withCRs
			lines: #(1)
			values: #(fileOutProtocol addProtocol renameProtocol removeProtocol)].
	^ ProtocolMenu!

removeProtocol
	| selectors |
	self changeRequest ifFalse: [^self].
	selectors := self selectedClass organization listAtCategoryNamed: protocol.
	selectors isEmpty ifFalse:
		[(self confirm: 'Are you certain that you want to
remove all methods in this protocol?') ifFalse: [^self].
		selectors do: [:sel | proto removeSelector: sel]].
	proto organization removeCategory: protocol.
	self newProtocolList: nil!

renameProtocol
	| aString newProtocol |
	self changeRequest ifFalse: [^self].
	aString := self prompt: 'Enter new protocol name' initially: protocol.
	aString isEmpty ifTrue: [^ self].
	newProtocol := aString asSymbol.
	(proto organization renameCategory: protocol to: newProtocol)
		ifTrue:
			[self newProtocolList: newProtocol]! !

!BasicProtoMethodBrowser methodsFor: 'initialize-release'!

on: aProto
	proto := aProto.
	category := nil.
	className := self className! !

!BasicProtoMethodBrowser methodsFor: 'selector messages'!

browseImplementors
	self error: 'not yet implemented'!

browseMessages
	self error: 'not yet implemented'!

browseSenders
	self error: 'not yet implemented'!

fileOutMessage
	| fileName sel fileStream |
	externalName isNil ifTrue: [self externalName: (self prompt: 'Type prototype external name' initially: '')].
	externalName isNil ifTrue: [^nil].
	selector isKeyword
		ifTrue: 
			[sel := WriteStream on: (String new: 20).
			selector keywords do: [:each | sel nextPutAll: (each copyFrom: 1 to: each size - 1)].
			sel := sel contents]
		ifFalse: [sel := selector].
	fileName := Filename
				request: 'File out on'
				initially: externalName , '-' , sel , '.st'
				shouldExist: false.
	fileName = '' ifTrue: [^nil].
	fileStream := (Filename named: fileName) writeStream.
	[proto
		fileOutMessage: selector
		file: fileStream
		name: externalName
		protocol: protocol]
		valueNowOrOnUnwindDo: [fileStream close]!

removeMethod
	(self changeRequest and: [self confirm: 'Are you certain that you
want to remove this method?'])
		ifTrue: 
			[proto removeMethod: selector.
			self newSelectorList: nil.
			^ true].
	^ false!

selectorMenu
	"Answer a Menu of operations on message selectors to be 
	displayed when the operate menu button is pressed."

	"Browser flushMenus"
	selector == nil ifTrue: [^ nil].
	MessageMenu == nil ifTrue:
		[MessageMenu := PopUpMenu
			labels: 'file out\senders\implementors\messages\remove' withCRs
			lines: #(1 4)
			values: #(fileOutMessage browseSenders browseImplementors browseMessages removeMethod)].
	^ MessageMenu! !

!BasicProtoMethodBrowser methodsFor: 'text'!

acceptMethod: aText from: aController
	|  newSelector |
	newSelector := proto method: aText string classified: protocol.
	newSelector == selector
		ifFalse: [self newSelectorList: newSelector].
	^true!

text
	| text method |
	textMode == #methodDefinition ifTrue: [selector == nil
			ifTrue: [^AbstractProto sourceCodeTemplate asText]
			ifFalse: 
				[method := proto compiledMethodAt: selector.
				(method isKindOf: CompiledMethod)
					ifTrue: [^'This is a primitive, \see Smalltalk Browser.' withCRs asText]
					ifFalse: [^method decompileString asText makeSelectorBoldIn: AbstractProto]]].
	textMode == #category ifTrue: [^'category to add' asText].
	textMode == #categories ifTrue: [^organization printString asText].
	textMode == #protocol ifTrue: [^'protocol to add' asText].
	textMode == #protocols ifTrue: [^self selectedClass organization printString asText].
	textMode == #comment
		ifTrue: 
			[text := self selectedClass comment asText.
			text isEmpty ifFalse: [^text].
			self selectedClass isMeta ifTrue: [^'Select the browser switch "instance" to see the comment' asText].
			^self selectedClass commentTemplateString asText].
	textMode == #hierarchy ifTrue: [^self selectedClass printHierarchy asText].
	^Text new! !

!BasicProtoMethodBrowser methodsFor: 'accessing'!

externalName: newName
	externalName := newName.
	self newClassList: newName.
	^newName! !

!BasicProtoMethodBrowser methodsFor: 'class messages'!

browseFieldReferences
	self error: 'not yet implemented'!

classMenu
	"Answer a Menu of operations on prototypes that is to be displayed 
	 when the operate menu button is pressed."
	"Browser flushMenus"

	ClassMenu == nil ifTrue: [ClassMenu := PopUpMenu
					labels: 'file out\protocols\inst var refs\find method' withCRs
					lines: #()
					values: #(#fileOutClass #editProtocols #browseFieldReferences #findMethodAndSelectAlphabetic)].
	^ClassMenu!

className
	^externalName isNil
		ifTrue: [proto printString]
		ifFalse: [externalName]!

fileOutClass

	| fileName  fileStream |
	externalName isNil ifTrue: [self externalName: (self prompt: 'Type prototype external name' initially: '')].
	fileName := Filename request: 'File out on' initially: (externalName , '.st') shouldExist: false. 
	fileName = '' ifTrue: [^nil].
	fileStream := (Filename named: fileName) writeStream.
	[proto
		fileOut: fileStream
		name: externalName]
		valueNowOrOnUnwindDo: [fileStream close]! !

BasicProtoMethodBrowser subclass: #ImplicitDelegationMethodBrowser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Prototypes-Tools'!


!ImplicitDelegationMethodBrowser methodsFor: 'prototype messages'!

browseParent! !

StringHolderView subclass: #ProtoSpaceView
	instanceVariableNames: 'toplevelObject '
	classVariableNames: 'ToplevelClass '
	poolDictionaries: ''
	category: 'Prototypes-Tools'!


!ProtoSpaceView methodsFor: 'controller access'!

defaultControllerClass
	"Answer the default controller class for the receiver."

	^ProtoSpaceController! !

!ProtoSpaceView methodsFor: 'initializing'!

initialize
	toplevelObject := ToplevelClass new.
	^super initialize! !

!ProtoSpaceView methodsFor: 'accessing'!

toplevelObject
	^toplevelObject!

toplevelObject: o
	toplevelObject := o! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

ProtoSpaceView class
	instanceVariableNames: ''!


!ProtoSpaceView class methodsFor: 'instance creation'!

openFor: aClass
	"Create a scheduled view of a workspace on the screen."
	ToplevelClass := aClass.
	self open: StringHolder new label: 'Prototype Workspace'!

openFor: aClass label: aString
	"Create a scheduled view of a workspace on the screen."
	ToplevelClass := aClass.
	self open: StringHolder new label: aString!

openFor: aClass label: aString contents: aText
	"Create a scheduled view of a workspace on the screen."
	ToplevelClass := aClass.
	self open: (StringHolder new contents: aText) label: aString! !

Dictionary variableSubclass: #PContext
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Prototypes-Tools'!


!PContext methodsFor: 'testing'!

hasLocalVar: name
	self at: name ifAbsent: [^false].
	^true! !

BrowserView subclass: #ProtoBrowserView
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Prototypes-Tools'!

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

ProtoBrowserView class
	instanceVariableNames: ''!


!ProtoBrowserView class methodsFor: 'instance creation'!

openOn: aProto 
	"BasicProtoMethodBrowserView openOn: Pen1"

	self openOn: (BasicProtoMethodBrowser new on: aProto) withTextState: nil!

openOn: aProto type: aBrowserClass
	"BasicProtoMethodBrowserView openOn: Pen1"

	self openOn: (aBrowserClass new on: aProto) withTextState: nil!

openOn: aProtoBrowser withTextState: anArray
	""

	| topView textView lineHeightBlock topWindow|
	lineHeightBlock := self lineHeightBlock.
	topWindow := ScheduledWindow
				model: aProtoBrowser
				label: aProtoBrowser printString
				minimumSize: 200@400.
	topView := self new.
	topWindow component: topView.
	topView
		addClassView: (LayoutFrame new
			leftOffset: 0;
			topOffset: 0;
			rightFraction: 0.50;
			bottomOffset: lineHeightBlock) on: aProtoBrowser readOnly: true;
		addProtocolView: (LayoutFrame new
			leftOffset: 0;
			topOffset: lineHeightBlock;
			rightFraction: 0.5;
			bottomFraction: 0.4) on: aProtoBrowser readOnly: false;
		addSelectorView: (LayoutFrame new
			leftFraction: 0.5;
			rightFraction: 1;
			bottomFraction: 0.4) on: aProtoBrowser readOnly: false.
	textView := CodeView
				on: aProtoBrowser
				aspect: #text
				change: #acceptText:from:
				menu: #textMenu
				initialSelection: nil.
	anArray == nil ifFalse: [self setTextView: textView fromTextState: anArray].
	topView
		add: (LookPreferences edgeDecorator on: textView)
		in: (0@0.4 corner: 1.0@1.0).
	topWindow icon: (Icon constantNamed: #protocolBrowser).
	topWindow open! !
